#include "strategies/strategy_base.hpp"

#include "strategies/priority.hpp"
#include "strategies/random1.hpp"

#include <climits>
#include <csignal>
#include <cstdio>
#include <cstdlib>
#include <vector>

const int INVALID_X_LENGTH = 1;
const int INVALID_X_RANGE = 2;
const int INVALID_Y_LENGTH = 3;
const int INVALID_Y_RANGE = 4;
const int QUERY_LIMIT_EXCEEDED = 5;
const int INVALID_A_LENGTH = 6;
const int INVALID_A_RANGE = 7;
const int WRONG_A = 8;
const int ANSWER_TWICE = 9;
const int NO_ANSWER = 10;

const std::vector<int> WA_FROM_STUB = {
    INVALID_X_LENGTH, INVALID_X_RANGE,  INVALID_Y_LENGTH,
    INVALID_Y_RANGE,  INVALID_A_LENGTH, INVALID_A_RANGE,
};

const int QUERY_LIMIT = 30;

FILE *in, *out;

void wrong(const int num) {
  fprintf(stderr, "translate:wrong\n");
  fprintf(stderr, "Wrong Answer [%d]\n", num);
  fprintf(stdout, "%.2lf\n", double(0));
  if (num)
    exit(0);
}

void ok() {
  fprintf(stderr, "translate:success\n");
  fprintf(stderr, "Accepted\n");
  fprintf(stdout, "%.2lf\n", double(1));
}

std::vector<int> read01seq(const int len) {
  std::vector<int> ret(len);
  for (int i = 0; i < len; i++) {
    if (fscanf(out, "%d", &ret[i]) != 1 || (ret[i] < 0 || 1 < ret[i])) {
      wrong(0);
      fprintf(stderr, "wrong format from stub.\n");
      exit(0);
    }
  }
  return ret;
}

int main(int argc, char **argv) {
  if (argc != 3) {
    fprintf(stderr, "Usage: %s <u-to-m> <m-to-u>\n", argv[0]);
    return 1;
  }

  char buf[256];
  scanf("%s", buf);
  std::string strategy_name(buf);
  strategy_base *strategy_ = nullptr;
  if (strategy_name == "random1") {
    strategy_ = new random1();
  } else if (strategy_name == "priority") {
    strategy_ = new priority();
  } else {
    wrong(0);
    fprintf(stderr, "unknown strategy.\n");
    return 0;
  }
  strategy_base &strategy = *strategy_;

  signal(SIGPIPE, SIG_IGN);
  out = fopen(argv[1], "r");
  in = fopen(argv[2], "w");

  const int N = strategy.N;
  if (!strategy.is_tree()) {
    wrong(0);
    fprintf(stderr, "not tree.\n");
    return 0;
  }

  fprintf(in, "%d ", N);
  for (int i = 0; i < N - 1; i++) {
    fprintf(in, "%d ", strategy.A[i]);
  }
  for (int i = 0; i < N - 1; i++) {
    fprintf(in, "%d%c", strategy.B[i], " \n"[i + 1 == N - 1]);
  }
  fflush(in);

  int query_count = 0;
  bool answered = false;
  while (true) {
    char c;
    if (fscanf(out, " %c", &c) != 1) {
      wrong(0);
      fprintf(stderr, "data not sent from stub.\n");
      return 0;
    }
    if (c == 'Q') {
      query_count++;
      if (query_count > QUERY_LIMIT) {
        fprintf(in, "-1\n");
        fflush(in);
        wrong(QUERY_LIMIT_EXCEEDED);
      }
      std::vector<int> x = read01seq(N - 1);
      std::vector<int> y = read01seq(N);
      std::vector<int> ret = strategy.Query(x, y);
      for (int i = 0; i < N; i++) {
        fprintf(in, "%d%c", ret[i], " \n"[i + 1 == N]);
      }
      fflush(in);
    } else if (c == 'A') {
      if (answered) {
        fprintf(in, "1\n");
        fflush(in);
        wrong(ANSWER_TWICE);
      }
      answered = true;
      std::vector<int> a = read01seq(N - 1);
      if (!strategy.Answer(a)) {
        fprintf(in, "1\n");
        fflush(in);
        wrong(WRONG_A);
      }
      fprintf(in, "0\n");
      fflush(in);
    } else if (c == 'F') {
      break;
    } else {
      for (const int e : WA_FROM_STUB) {
        if (c == '0' + e) {
          wrong(e);
        }
      }
      wrong(0);
      fprintf(stderr, "wrong character: %c\n", c);
      return 0;
    }
  }
  if (!answered) {
    wrong(NO_ANSWER);
  }
  ok();
  return 0;
}