#include "conveyor.h"

#include <csignal>
#include <cstdio>
#include <cstdlib>
#include <utility>
#include <vector>

namespace {

FILE *in, *out;
int N;

const int INVALID_X_LENGTH = 1;
const int INVALID_X_RANGE = 2;
const int INVALID_Y_LENGTH = 3;
const int INVALID_Y_RANGE = 4;
const int QUERY_LIMIT_EXCEEDED = 5;
const int INVALID_A_LENGTH = 6;
const int INVALID_A_RANGE = 7;
const int WRONG_A = 8;
const int ANSWER_TWICE = 9;
const int NO_ANSWER = 10;

} // namespace

std::vector<int> Query(std::vector<int> x, std::vector<int> y) {
  if (int(x.size()) != N - 1) {
    fprintf(out, "%d\n", INVALID_X_LENGTH);
    fflush(out);
    exit(0);
  }
  for (const int &v : x) {
    if (v < 0 || 1 < v) {
      fprintf(out, "%d\n", INVALID_X_RANGE);
      fflush(out);
      exit(0);
    }
  }

  if (int(y.size()) != N) {
    fprintf(out, "%d\n", INVALID_Y_LENGTH);
    fflush(out);
    exit(0);
  }
  for (const int &v : y) {
    if (v < 0 || 1 < v) {
      fprintf(out, "%d\n", INVALID_Y_RANGE);
      fflush(out);
      exit(0);
    }
  }

  fprintf(out, "Q\n");
  for (int i = 0; i < N - 1; i++) {
    fprintf(out, "%d%c", x[i], " \n"[i + 1 == N - 1]);
  }
  for (int i = 0; i < N; i++) {
    fprintf(out, "%d%c", y[i], " \n"[i + 1 == N]);
  }
  fflush(out);

  std::vector<int> ret(N);
  for (int i = 0; i < N; i++) {
    if (fscanf(in, "%d", &ret[i]) != 1) {
      fprintf(stderr, "data not sent from manager.\n");
      exit(1);
    }
    if (i == 0 && ret[i] == -1) {
      exit(0);
    }
    if (ret[i] < 0 || 1 < ret[i]) {
      fprintf(stderr, "wrong data.\n");
      exit(1);
    }
  }
  return ret;
}

void Answer(std::vector<int> a) {
  if (int(a.size()) != N - 1) {
    fprintf(out, "%d\n", INVALID_A_LENGTH);
    fflush(out);
    exit(0);
  }
  for (const int &v : a) {
    if (v < 0 || 1 < v) {
      fprintf(out, "%d\n", INVALID_A_RANGE);
      fflush(out);
      exit(0);
    }
  }

  fprintf(out, "A\n");
  for (int i = 0; i < N - 1; i++) {
    fprintf(out, "%d%c", a[i], " \n"[i + 1 == N - 1]);
  }
  fflush(out);

  int ret;
  if (fscanf(in, "%d", &ret) != 1) {
    fprintf(stderr, "data not sent from manager.\n");
    exit(1);
  }
  if (ret != 0) {
    exit(0);
  }
}

int main(int argc, char **argv) {
  if (argc != 3) {
    fprintf(stderr, "Usage: %s <m-to-u> <u-to-m>\n", argv[0]);
    return 1;
  }
  // open pipe
  signal(SIGPIPE, SIG_IGN);
  out = fopen(argv[2], "w");
  in = fopen(argv[1], "r");

  if (fscanf(in, "%d", &N) != 1) {
    fprintf(stderr, "data not sent from manager.\n");
    return 1;
  }
  std::vector<int> A(N - 1), B(N - 1);
  for (int i = 0; i < N - 1; i++) {
    if (fscanf(in, "%d", &A[i]) != 1) {
      fprintf(stderr, "data not sent from manager.\n");
      return 1;
    }
  }
  for (int i = 0; i < N - 1; i++) {
    if (fscanf(in, "%d", &B[i]) != 1) {
      fprintf(stderr, "data not sent from manager.\n");
      return 1;
    }
  }
  Solve(N, std::move(A), std::move(B));
  fprintf(out, "F\n");
  fflush(out);
  return 0;
}